package org.gcube.portlets.widgets.ckancontentmoderator.client;

import org.gcube.datacatalogue.utillibrary.shared.ItemStatus;
import org.gcube.portlets.widgets.ckancontentmoderator.client.ui.MainPanel;
import org.gcube.portlets.widgets.ckancontentmoderator.client.ui.table.ItemsTable.DISPLAY_FIELD;
import org.gcube.portlets.widgets.ckancontentmoderator.client.ui.util.LoadingPanel;
import org.gcube.portlets.widgets.ckancontentmoderator.client.ui.util.UtilUx;

import com.github.gwtbootstrap.client.ui.constants.AlertType;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.ui.ComplexPanel;
import com.google.gwt.user.client.ui.HTML;

/**
 * The Class CheckConfigsUxController.
 *
 * @author Francesco Mangiacrapa at ISTI-CNR francesco.mangiacrapa@isti.cnr.it
 * 
 *         Apr 8, 2022
 */
public class CheckConfigsUxController {

	private LoadingPanel loader;

	private CkanContentModeratorCheckConfigs moderatorCheckConfig;

	private ComplexPanel basePanelContainer;

	private CkanContentModeratorWidgetController cmsController;

	private MainPanel mainPanel;

	/**
	 * Instantiates a new check configs ux controller.
	 *
	 * @param baseCP               the base CP
	 * @param moderatorCheckConfig the moderatorcheck config
	 */
	protected CheckConfigsUxController(MainPanel mainPanel, CkanContentModeratorCheckConfigs moderatorcheckConfig) {
		this.mainPanel = mainPanel;
		this.basePanelContainer = mainPanel.getMainPanelContainer();
		this.moderatorCheckConfig = moderatorcheckConfig;
		checkConfigs();
	}

	/**
	 * Check configs.
	 */
	private void checkConfigs() {

		if (moderatorCheckConfig == null) {
			// here the moderatorCheckConfig is null, so loading configurations from server
			// and
			// checking them
			loader = new LoadingPanel(new HTML("Checking configurations and authorizations... please wait"));
			basePanelContainer.add(loader);
			moderatorCheckConfig = new CkanContentModeratorCheckConfigs();

			final Command whenDone = new Command() {

				@Override
				public void execute() {
					onConfigurationsLoaded();
				}
			};

			try {
				moderatorCheckConfig.checkConfigs(whenDone, false);
			} catch (Exception e) {
				GWT.log("Check configs error: " + e.getMessage());
			}
		} else {
			// here the moderatorCheckConfig is already full
			onConfigurationsLoaded();
		}
	}

	/**
	 * On configurations loaded.
	 */
	private void onConfigurationsLoaded() {
		GWT.log("onConfigurationLoaded executed");
		boolean isContentModerationEnabled = false;
		boolean isModeratorRoleAssingned = false;
		boolean isExistsMyItemInModeration = false;

		try {
			isContentModerationEnabled = moderatorCheckConfig.isContentModerationEnabled();
		} catch (Exception e) {
			GWT.log("Command - Check configs error: " + e.getMessage());
		}

		try {
			isModeratorRoleAssingned = moderatorCheckConfig.isModeratorRoleAssigned();
		} catch (Exception e) {
			GWT.log("Command - Check configs error: " + e.getMessage());
		}
		try {
			isExistsMyItemInModeration = moderatorCheckConfig.isExistsMyItemInModeration();
		} catch (Exception e) {
			GWT.log("Command - Check configs error: " + e.getMessage());
		}

		GWT.log("Moderation is enabled? " + isContentModerationEnabled);
		GWT.log("Moderator role is assigned? " + isModeratorRoleAssingned);
		GWT.log("isExistsMyItemInModeration? " + isExistsMyItemInModeration);

		if (!isContentModerationEnabled) {
			try {
				basePanelContainer.remove(loader);
			} catch (Exception e) {
			}
			UtilUx.showAlert("The Moderation facility is not enabled in this context", AlertType.WARNING, false,
					basePanelContainer, null);
			return;
		}

		// moderator logged in
		if (isModeratorRoleAssingned) {
			try {
				basePanelContainer.remove(loader);
			} catch (Exception e) {
			}
			cmsController = new CkanContentModeratorWidgetController(ItemStatus.PENDING, DISPLAY_FIELD.values(),
					DISPLAY_FIELD.NAME, false, false);
			basePanelContainer.add(cmsController.getMainPanel());

			mainPanel.setLoggedLabelText("Logged in as Moderator");
			return;
		}

		// no item under moderation or already moderated
		if (!isExistsMyItemInModeration) {
			try {
				basePanelContainer.remove(loader);
			} catch (Exception e) {
			}
			UtilUx.showAlert("You are not authorized to access to Moderation facility", AlertType.WARNING, false,
					basePanelContainer, null);
			return;
		}

		try {
			basePanelContainer.remove(loader);
		} catch (Exception e) {
		}

		// read only mode enabled in case of moderator role not assigned
		boolean readOnlyMode = !isModeratorRoleAssingned;
		// Listing only data belonging to user logged in. He/She must be the owner of.
		boolean restrictDataToLoggedInUser = readOnlyMode;
		
		GWT.log("readOnlyMode is enabled? " + readOnlyMode);
		GWT.log("restrictDataToLoggedInUser is? " + restrictDataToLoggedInUser);

		cmsController = new CkanContentModeratorWidgetController(ItemStatus.PENDING, DISPLAY_FIELD.values(),
				DISPLAY_FIELD.NAME, readOnlyMode, restrictDataToLoggedInUser);

		basePanelContainer.add(cmsController.getMainPanel());

	}

	public CkanContentModeratorCheckConfigs getModeratorCheckConfig() {
		return moderatorCheckConfig;
	}

}
